@@ -37,12 +37,22 @@ namespace Beagle.Daemon {
 
 	public class FileAttributesStore_Sqlite : IFileAttributesStore {
 
-		const int VERSION = 1;
+		// Version history:
+		// 1: Original version
+		// 2: Replaced LastIndexedTime with LastAttrTime
+		const int VERSION = 2;
 
 		private SqliteConnection connection;
 		private BitArray path_flags;
 		private int transaction_count = 0;
 
+		enum TransactionState {
+			None,
+			Requested,
+			Started
+		}
+		private TransactionState transaction_state;
+
 		public FileAttributesStore_Sqlite (string directory, string index_fingerprint)
 		{
 			bool create_new_db = false;
@@ -51,35 +61,54 @@ namespace Beagle.Daemon {
 			if (! File.Exists (GetDbPath (directory))) {
 				create_new_db = true;
 			} else {
-				connection = Open (directory);
+				
+				// Funky logic here to deal with sqlite versions.
+				//
+				// When sqlite 3 tries to open an sqlite 2 database,
+				// it will throw an SqliteException with SqliteError
+				// NOTADB when trying to execute a command.
+				//
+				// When sqlite 2 tries to open an sqlite 3 database,
+				// it will throw an ApplicationException when it
+				// tries to open the database.
 
-				SqliteCommand command;
-				SqliteDataReader reader = null;
-				int stored_version = 0;
-				string stored_fingerprint = null;
-
-
-				command = new SqliteCommand ();
-				command.Connection = connection;
-				command.CommandText =
-					"SELECT version, fingerprint FROM db_info";
 				try {
-					reader = ExecuteReaderOrWait (command);
-				} catch (Exception ex) {
+					connection = Open (directory);
+				} catch (ApplicationException) {
+					Logger.Log.Warn ("Likely sqlite database version mismatch trying to open {0}.  Purging.", GetDbPath (directory));
 					create_new_db = true;
 				}
-				if (reader != null && ! create_new_db) {
-					if (ReadOrWait (reader)) {
-						stored_version = reader.GetInt32 (0);
-						stored_fingerprint = reader.GetString (1);
-					}
-					reader.Close ();
-				}
-				command.Dispose ();
 
-				if (VERSION != stored_version
-				    || (index_fingerprint != null && index_fingerprint != stored_fingerprint))
-					create_new_db = true;
+				if (! create_new_db) {
+					SqliteCommand command;
+					SqliteDataReader reader = null;
+					int stored_version = 0;
+					string stored_fingerprint = null;
+
+
+					command = new SqliteCommand ();
+					command.Connection = connection;
+					command.CommandText =
+						"SELECT version, fingerprint FROM db_info";
+					try {
+						reader = ExecuteReaderOrWait (command);
+					} catch (Exception ex) {
+						Logger.Log.Warn ("Likely sqlite database version mismatch trying to read from {0}.  Purging.", GetDbPath (directory));
+						create_new_db = true;
+					}
+					if (reader != null && ! create_new_db) {
+						if (ReadOrWait (reader)) {
+							stored_version = reader.GetInt32 (0);
+							stored_fingerprint = reader.GetString (1);
+						}
+						reader.Close ();
+					}
+					command.Dispose ();
+
+					if (VERSION != stored_version
+					    || (index_fingerprint != null && index_fingerprint != stored_fingerprint))
+						create_new_db = true;
+				}
 			}
 
 			if (create_new_db) {
@@ -101,7 +130,7 @@ namespace Beagle.Daemon {
 					    "  directory      STRING NOT NULL,        " +
 					    "  filename       STRING NOT NULL,        " +
 					    "  last_mtime     STRING NOT NULL,        " +
-					    "  last_indexed   STRING NOT NULL,        " +
+					    "  last_attrtime  STRING NOT NULL,        " +
 					    "  filter_name    STRING NOT NULL,        " +
 					    "  filter_version STRING NOT NULL         " +
 					    ")");
@@ -136,8 +165,6 @@ namespace Beagle.Daemon {
 				Logger.Log.Debug ("Loaded {0} records from {1} in {2:0.000}s", 
 						 count, GetDbPath (directory), (dt2 - dt1).TotalSeconds);
 			}
-
-			Shutdown.ShutdownEvent += Flush;
 		}
 
 		///////////////////////////////////////////////////////////////////
@@ -151,7 +178,8 @@ namespace Beagle.Daemon {
 		{
 			SqliteConnection c;			
 			c = new SqliteConnection ();
-			c.ConnectionString = "URI=file:" + GetDbPath (directory);
+			c.ConnectionString = "version=" + ExternalStringsHack.SqliteVersion
+				+ ",encoding=UTF-8,URI=file:" + GetDbPath (directory);
 			c.Open ();
 			return c;
 		}
@@ -167,11 +195,8 @@ namespace Beagle.Daemon {
 				try {
 					command.ExecuteNonQuery ();
 					break;
-				} catch (SqliteException ex) {
-					if (ex.SqliteError == SqliteError.BUSY)
-						Thread.Sleep (50);
-					else
-						throw ex;
+				} catch (SqliteBusyException ex) {
+					Thread.Sleep (50);
 				}
 			}
 
@@ -184,7 +209,7 @@ namespace Beagle.Daemon {
 			command = new SqliteCommand ();
 			command.Connection = connection;
 			command.CommandText =
-				"SELECT unique_id, directory, filename, last_mtime, last_indexed, filter_name, filter_version " +
+				"SELECT unique_id, directory, filename, last_mtime, last_attrtime, filter_name, filter_version " +
 				"FROM file_attributes WHERE " + 
 				String.Format (where_format, where_args);
 			return command;
@@ -196,11 +221,8 @@ namespace Beagle.Daemon {
 			while (reader == null) {
 				try {
 					reader = command.ExecuteReader ();
-				} catch (SqliteException ex) {
-					if (ex.SqliteError == SqliteError.BUSY)
-						Thread.Sleep (50);
-					else
-						throw ex;
+				} catch (SqliteBusyException ex) {
+					Thread.Sleep (50);
 				}
 			}
 			return reader;
@@ -211,11 +233,8 @@ namespace Beagle.Daemon {
 			while (true) {
 				try {
 					return reader.Read ();
-				} catch (SqliteException ex) {
-					if (ex.SqliteError == SqliteError.BUSY)
-						Thread.Sleep (50);
-					else
-						throw ex;
+				} catch (SqliteBusyException ex) {
+					Thread.Sleep (50);
 				}
 			}
 		}
@@ -227,10 +246,13 @@ namespace Beagle.Daemon {
 			attr.UniqueId = GuidFu.FromShortString (reader.GetString (0));
 			attr.Path = System.IO.Path.Combine (reader.GetString (1), reader.GetString (2));
 			attr.LastWriteTime = StringFu.StringToDateTime (reader.GetString (3));
-			attr.LastIndexedTime = StringFu.StringToDateTime (reader.GetString (4));
+			attr.LastAttrTime = StringFu.StringToDateTime (reader.GetString (4));
 			attr.FilterName = reader.GetString (5);
 			attr.FilterVersion = int.Parse (reader.GetString (6));
 
+			if (attr.FilterName == "")
+				attr.FilterName = null;
+
 			return attr;
 		}
 
@@ -272,7 +294,7 @@ namespace Beagle.Daemon {
 
 			// We need to quote any 's that appear in the strings
 			// (int particular, in the path)
-			string directory = Path.GetDirectoryName (path).Replace ("'", "''");
+			string directory = FileSystem.GetDirectoryNameRootOk (path).Replace ("'", "''");
 			string filename = Path.GetFileName (path).Replace ("'", "''");
 			lock (connection) {
 				command = QueryCommand ("directory='{0}' AND filename='{1}'",
@@ -309,30 +331,25 @@ namespace Beagle.Daemon {
 			// We need to quote any 's that appear in the strings
 			// (in particular, in the path)
 			lock (connection) {
-				if (transaction_count == 0 && ! Shutdown.ShutdownRequested) {
-					Logger.Log.Debug ("Beginning sqlite transaction");
-					DoNonQuery ("BEGIN");
-				}
+				
+				// If a transaction has been requested, start it now.
+				MaybeStartTransaction ();
+
+				string filter_name;
+				filter_name = fa.FilterName;
+				if (filter_name == null)
+					filter_name = "";
+				filter_name = filter_name.Replace ("'", "''");
 
 				DoNonQuery ("INSERT OR REPLACE INTO file_attributes " +
-					    " (unique_id, directory, filename, last_mtime, last_indexed, filter_name, filter_version) " +
+					    " (unique_id, directory, filename, last_mtime, last_attrtime, filter_name, filter_version) " +
 					    " VALUES ('{0}', '{1}', '{2}', '{3}', '{4}', '{5}', '{6}')",
 					    GuidFu.ToShortString (fa.UniqueId),
 					    fa.Directory.Replace ("'", "''"), fa.Filename.Replace ("'", "''"),
 					    StringFu.DateTimeToString (fa.LastWriteTime),
-					    StringFu.DateTimeToString (fa.LastIndexedTime),
-					    fa.FilterName,
+					    StringFu.DateTimeToString (fa.LastAttrTime),
+					    filter_name,
 					    fa.FilterVersion);
-
-				if (! Shutdown.ShutdownRequested)
-					++transaction_count;
-
-				// 150 is a pretty arbitrary number
-				if (transaction_count == 150) {
-					Logger.Log.Debug ("Committing sqlite transaction");
-					DoNonQuery ("COMMIT");
-					transaction_count = 0;
-				}
 			}
 			return true;
 		}
@@ -344,14 +361,41 @@ namespace Beagle.Daemon {
 
 			// We need to quote any 's that appear in the strings
 			// (in particular, in the path)
-			string directory = Path.GetDirectoryName (path).Replace ("'", "''");
+			string directory = FileSystem.GetDirectoryNameRootOk (path).Replace ("'", "''");
 			string filename = Path.GetFileName (path).Replace ("'", "''");
 			lock (connection) {
+
+				// If a transaction has been requested, start it now.
+				MaybeStartTransaction ();
+
 				DoNonQuery ("DELETE FROM file_attributes WHERE directory='{0}' AND filename='{1}'",
 					    directory, filename);
 			}
 		}
 
+		private void MaybeStartTransaction ()
+		{
+			if (transaction_state == TransactionState.Requested) {
+				DoNonQuery ("BEGIN");
+				transaction_state = TransactionState.Started;
+			}
+		}
+
+		public void BeginTransaction ()
+		{
+			if (transaction_state == TransactionState.None)
+				transaction_state = TransactionState.Requested;
+		}
+
+		public void CommitTransaction ()
+		{
+			if (transaction_state == TransactionState.Started) {
+				lock (connection)
+					DoNonQuery ("COMMIT");
+			}
+			transaction_state = TransactionState.None;
+		}
+
 		public void Flush ()
 		{
 			lock (connection) {
@@ -374,14 +418,12 @@ namespace Beagle.Daemon {
 			SqliteCommand command;
 			SqliteDataReader reader;
 				
-			FileAttributes attr = null;
-			
 			lock (connection) {
 				command = new SqliteCommand ();
 				command.Connection = connection;
 				command.CommandText =
-					"SELECT unique_id, directory, filename, last_mtime, last_indexed, filter_name, filter_version " +
-					"FROM file_attributes"; 
+					"SELECT unique_id, directory, filename, last_mtime, last_attrtime, filter_name, filter_version " +
+					"FROM file_attributes";
 				
 				reader = ExecuteReaderOrWait (command);
 				
@@ -401,8 +443,12 @@ namespace Beagle.Daemon {
 		{
 			ICollection attributes = fa_sqlite_store_to_merge.ReadAllAttributes ();
 				
+			BeginTransaction ();
+
 			foreach (FileAttributes attribute in attributes)
 				Write (attribute);
+
+			CommitTransaction ();
 		}
 	}
 }
