@@ -38,6 +38,16 @@ namespace Beagle.Daemon {
 
 		public delegate string StringSource ();
 
+		static private bool IsTokenSeparator (char c)
+		{
+			return Char.IsWhiteSpace (c)
+				|| Char.IsSeparator (c)
+				|| Char.IsPunctuation (c);
+		}
+
+		const int max_prior_words = 6;
+		const int max_following_words = 6;
+
 		static private void HighlightTerms (string [] stemmed_terms, string text, ref ArrayList matches)
 		{
 			int pos = 0, prev_stop_pos = 0, prev_end_pos = 0;
@@ -46,52 +56,59 @@ namespace Beagle.Daemon {
 			while (pos < text.Length) {
 				
 				// Find the beginning of the next token.
-				if (! Char.IsLetterOrDigit (text [pos])) {
+				if (IsTokenSeparator (text [pos])) {
 					++pos;
 					continue;
 				}
-
+				
 				// Find the end of the next token
 				int next_pos = pos+1;
-				while (next_pos < text.Length && Char.IsLetterOrDigit (text [next_pos]))
+				while (next_pos < text.Length && !IsTokenSeparator (text [next_pos]))
 					++next_pos;
-
+				
 				string stemmed_token = null;
-
+				int hl_offset = 0;
+				
 				// Iterate through the stemmed terms and match the token
 				for (int i = 0; i < stemmed_terms.Length; i++) {
-
+					
 					// If this term is longer than the token in question, give up.
 					if (next_pos - pos < stemmed_terms [i].Length)
 						continue;
+					
+					// Make sure this isn't a stop word.
+					if (LuceneCommon.IsStopWord (stemmed_terms [i]))
+						continue;
 
 					// We cache the token, so as to avoid stemming it more than once
 					// when considering multiple terms.
 					if (stemmed_token == null) {
 						string token = text.Substring (pos, next_pos - pos).ToLower ();
-						stemmed_token = LuceneDriver.Stem (token);
+						stemmed_token = LuceneCommon.Stem (token);
 					}
 
 					if (stemmed_terms [i] != stemmed_token)
 						continue;
-
+					
 					// We have a match!
 
 				        int start_pos = pos;
 					int stop_pos = next_pos;
 
 					// FIXME: This is a hack, I should be shot.
-					for (int count = 0; count < 3 && start_pos > 0; start_pos--) {
+					for (int count = 0; count <= max_prior_words && start_pos > 0; start_pos--) {
 						if ((text[start_pos] == ' '))
 							count++;
 					}
+
 					if (start_pos != 0)
 						start_pos += 2;
 
-					for (int count = 0; count < 3 && stop_pos < text.Length; stop_pos++) {
+					for (int count = 0; count <= max_following_words && stop_pos < text.Length; stop_pos++) {
 						if (text[stop_pos] == ' ')
 							count++;
 					}
+
 					if (stop_pos != text.Length)
 						stop_pos--;
 
@@ -105,7 +122,7 @@ namespace Beagle.Daemon {
 
 					string new_match = String.Concat (text.Substring (start_pos, pos - start_pos),
 									  "<font color=\"",
-									  colors [i%colors.Length],
+									  colors [(i - hl_offset) % colors.Length],
 									  "\"><b>",
 									  text.Substring (pos, next_pos-pos),
 									  "</b></font>",
@@ -121,7 +138,7 @@ namespace Beagle.Daemon {
 
 					prev_stop_pos = stop_pos;
 					prev_end_pos = next_pos;
-					
+
 					break;
 				}
 
@@ -131,15 +148,13 @@ namespace Beagle.Daemon {
 			// Add trailing match
 			if (prev_match != "")
 				matches.Add (prev_match); 
-
-			return;
 		}
 
-		static string[] colors = new string [] { "red", "blue", "green", "orange", "purple", "brown"};
+		static string[] colors = new string [] {"red", "blue", "green", "orange", "purple", "brown"};
 
 		const int soft_snippet_limit = 400;
 
-		static public string GetSnippet (QueryBody    query_body,
+		static public string GetSnippet (string[]     query_terms,
 						 StringSource string_source)
 		{
 			// FIXME: If the query doesn't have search text (or is null), we should
@@ -147,22 +162,12 @@ namespace Beagle.Daemon {
 
 			if (string_source == null)
 				return null;
-
-			IList query_terms = query_body.Text;
-			int N = query_terms.Count;
-			string[] stemmed_terms = new string [N];
-			for (int i = 0; i < N; ++i) {
-				string term = (string) query_terms [i];
-				if (term [0] == '-')
-					continue;
-				stemmed_terms [i] = LuceneDriver.Stem ((string) query_terms [i]).ToLower ();
-			}
 			
 			ArrayList matches = new ArrayList ();
 
 			string str;
 			while ( (str = string_source ()) != null) {
-				HighlightTerms (stemmed_terms, str, ref matches);
+				HighlightTerms (query_terms, str, ref matches);
 			}
 
 			string snippet = "";
@@ -174,16 +179,18 @@ namespace Beagle.Daemon {
 		
 		}
 		
-		static public string GetSnippet (QueryBody  body,
+		static public string GetSnippet (string[]   query_terms,
 						 TextReader reader)
 		{
-			return GetSnippet (body, new StringSource (reader.ReadLine));
+			return GetSnippet (query_terms, new StringSource (reader.ReadLine));
 		}
 
-		static public string GetSnippetFromFile (QueryBody body,
-							 string    filename)
+		static public string GetSnippetFromFile (string[] query_terms,
+							 string   filename)
 		{
-			return GetSnippet (body, new StreamReader (filename));
+			FileStream stream = new FileStream (filename, FileMode.Open, FileAccess.Read, FileShare.ReadWrite);
+
+			return GetSnippet (query_terms, new StreamReader (stream));
 		}
 	}
 
