@@ -39,7 +39,11 @@ screensaver_info (int *state, int *kind, unsigned long *til_or_since, unsigned l
     XScreenSaverInfo ss_info;
     int retval;
     static int inited = 0;
+    int event_base, error_base;
 
+    if (GDK_DISPLAY () == NULL)
+        return 0;
+	
     /* FIXME: This should be called somewhere else. */
     if (! inited) {
         gdk_threads_init ();
@@ -47,7 +51,10 @@ screensaver_info (int *state, int *kind, unsigned long *til_or_since, unsigned l
     }
 
     gdk_threads_enter ();
-    retval = XScreenSaverQueryInfo (GDK_DISPLAY (), DefaultRootWindow (GDK_DISPLAY ()), &ss_info);
+    if (XScreenSaverQueryExtension (GDK_DISPLAY (), &event_base, &error_base))
+        retval = XScreenSaverQueryInfo (GDK_DISPLAY (), DefaultRootWindow (GDK_DISPLAY ()), &ss_info);
+    else
+        retval = 0;
     gdk_threads_leave ();
 
     if (retval != 0) {
