@@ -1,8 +1,8 @@
 using System;
 using System.Diagnostics;
-using Mono.Posix;
+using Mono.Unix;
 
-using BU = Beagle.Util;
+using Beagle.Util;
 
 namespace Search.Tiles {
 
@@ -26,7 +26,6 @@ namespace Search.Tiles {
 			Group = TileGroup.Image;
 
 			Title = Hit ["beagle:ExactFilename"];
-			Icon = GetIcon (Hit, 32);
 
 			if (Hit ["beagle:FilenameExtension"].Length > 0)
 				Description = Hit ["beagle:FilenameExtension"].Substring (1).ToUpper ();
@@ -34,67 +33,55 @@ namespace Search.Tiles {
 			if (Hit ["fixme:width"] != null && Hit ["fixme:width"] != "")
 				Description += String.Format (" {0}x{1}", Hit ["fixme:width"], Hit ["fixme:height"]);
 
-			Description += String.Format (" ({0})", BU.StringFu.FileLengthToString (Hit.FileInfo.Length));
+			Description += String.Format (" ({0})", StringFu.FileLengthToString (Hit.FileInfo.Length));
 
-			AddAction (new TileAction (Catalog.GetString ("Add to Library"), Gtk.Stock.Add, AddToLibrary));
-			AddAction (new TileAction (Catalog.GetString ("Set as Wallpaper"), SetAsWallpaper)); // FIXME: This is not in the spec, is it ok?
+			// AddAction (new TileAction (Catalog.GetString ("Add to Library"), Gtk.Stock.Add, AddToLibrary));
+			AddAction (new TileAction (Catalog.GetString ("Set as Wallpaper"), SetAsWallpaper));
 		}
 
-		public static Gdk.Pixbuf GetIcon (Beagle.Hit hit, int size)
+		protected override void LoadIcon (Gtk.Image image, int size)
 		{
-			Gdk.Pixbuf icon =  WidgetFu.LoadThumbnailIcon (hit.Uri, size);
-
-			if (icon == null)
-				return WidgetFu.LoadMimeIcon (hit ["beagle:MimeType"], size);
+			base.LoadIcon (image, size);
 
 			// Draw the F-Spot overlay
-			if (hit ["fspot:IsIndexed"] == "true") {
+			if (Hit ["fspot:IsIndexed"] == "true") {
+				Gdk.Pixbuf icon = image.Pixbuf;
+
 				Gdk.Pixbuf emblem = Beagle.Images.GetPixbuf ("emblem-fspot.png", 16, 16);
 				emblem.Composite (icon, 0,  icon.Height - emblem.Height, emblem.Width,
 						  emblem.Height, 0,  icon.Height - emblem.Height, 1,  1,
 						  Gdk.InterpType.Bilinear, 255);
-			}
 
-			return icon;
+				image.Pixbuf = icon;
+			}
 		}
 
-		const Gtk.AttachOptions expand = Gtk.AttachOptions.Expand | Gtk.AttachOptions.Fill;
-		const Gtk.AttachOptions fill = Gtk.AttachOptions.Fill;
-
-		protected override Gtk.Widget GetDetails ()
+		protected override DetailsPane GetDetails ()
 		{
-			Gtk.Table table = new Gtk.Table (3, 4, false);
-			table.RowSpacing = table.ColumnSpacing = 6;
+			DetailsPane details = new DetailsPane ();
 
-			// FIXME: The icon needs a nice frame as in the spec
-			Gtk.Image icon = new Gtk.Image (GetIcon (Hit, 96));
-			table.Attach (icon, 0, 1, 0, 4, fill, fill, 0, 0);
+			// FIXME: The icon needs a nice frame as in the spec (?)
 
-			Gtk.Label label;
-			label = WidgetFu.NewBoldLabel (Title);
-			table.Attach (label, 1, 3, 0, 1, expand, fill, 0, 0);
-			
-			label = WidgetFu.NewLabel (Description);
-			table.Attach (label, 1, 3, 1, 2, expand, fill, 0, 0);
+			details.AddBoldLabel (Title, 0, 1);
+			details.AddLabel (Description, 1, 1);
+			details.AddLabel ("", 2, 1);
 
-			label = WidgetFu.NewGrayLabel (Catalog.GetString ("Modified:"));
-			table.Attach (label, 1, 2, 2, 3, fill, fill, 0, 0);
-
-			label = WidgetFu.NewLabel (Utils.NiceVeryLongDate (Hit.FileInfo.LastWriteTime));
-			table.Attach (label, 2, 3, 2, 3, expand, fill, 0, 0);			
+			details.AddLabelPair (Catalog.GetString ("Modified:"),
+					      Utils.NiceVeryLongDate (Hit.FileInfo.LastWriteTime),
+					      3, 1);
+			details.AddLabelPair (Catalog.GetString ("Full Path:"),
+					      Hit.Uri.LocalPath,
+					      4, 1);
 
 			if (Hit ["fspot:Description"] != null && Hit ["fspot:Description"] != "") {
-				label = WidgetFu.NewLabel (Hit ["fspot:Description"]);
-				WidgetFu.EllipsizeLabel (label);
-				table.Attach (label, 1, 3, 3, 4, expand, expand, 48, 0);
+				details.AddLabel ("", 5, 1);
+				details.AddLabel (Hit ["fspot:Description"], 6, 1);
 			}
 
-			table.WidthRequest = 0;
-			table.ShowAll ();
-			
-			return table;
+			return details;
 		}
 		
+#if NOT_YET
 		// FIXME: fspot doesnt allow to import a particular file
 		// only a whole directory
 		public void AddToLibrary ()
@@ -108,7 +95,7 @@ namespace Search.Tiles {
 			pi.Arguments = String.Format ("--import {0}", Hit.FileInfo.FullName);
 			Process.Start (pi);
 		}
-
+#endif
 		
 		public void SetAsWallpaper ()
 		{
